@@ -1,14 +1,10 @@
-#undef HAVE_BROKEN_READDIR
-#undef HAVE_ERRNO_DECL
-#undef HAVE_LONGLONG
-#undef HAVE_OFF64_T
-#undef HAVE_REMSH
-#undef HAVE_UNSIGNED_CHAR
-#undef HAVE_UTIMBUF
 #undef ino_t
 #undef HAVE_CONNECT
 #undef HAVE_SHORT_INO_T
 #undef HAVE_GETOPT_LONG
 #undef REPLACE_INET_NTOA
+#undef REPLACE_INET_ATON
 #undef HAVE_GETTIMEOFDAY_TZ
+#undef ENABLE_IPV6
+#undef HAVE_SOCKADDR_LEN
 #undef HAVE_SOCKETPAIR
