@@ -1,6 +1,7 @@
-/* 
-   Copyright (C) Andrew Tridgell 1996
-   Copyright (C) Paul Mackerras 1996
+/* -*- c-file-style: "linux" -*-
+     
+   Copyright (C) 1996-2001 by Andrew Tridgell <tridge@samba.org>
+   Copyright (C) 1996 by Paul Mackerras
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +21,8 @@
 /* a lot of this stuff was originally derived from GNU tar, although
    it has now changed so much that it is hard to tell :) */
 
+/* include/exclude cluestick added by Martin Pool <mbp@samba.org> */
+
 #include "rsync.h"
 
 extern int verbose;
@@ -28,7 +31,7 @@ extern int delete_mode;
 static struct exclude_struct **exclude_list;
 
 /* build an exclude structure given a exclude pattern */
-static struct exclude_struct *make_exclude(char *pattern, int include)
+static struct exclude_struct *make_exclude(const char *pattern, int include)
 {
 	struct exclude_struct *ret;
 
@@ -84,8 +87,8 @@ static void free_exclude(struct exclude_struct *ex)
 	free(ex);
 }
 
-static int check_one_exclude(char *name,struct exclude_struct *ex,
-			     STRUCT_STAT *st)
+static int check_one_exclude(char *name, struct exclude_struct *ex,
+                             STRUCT_STAT *st)
 {
 	char *p;
 	int match_start=0;
@@ -121,32 +124,62 @@ static int check_one_exclude(char *name,struct exclude_struct *ex,
 }
 
 
-int check_exclude(char *name,struct exclude_struct **local_exclude_list,
+static void report_exclude_result(char const *name,
+                                  struct exclude_struct const *ent,
+                                  STRUCT_STAT const *st)
+{
+        /* If a trailing slash is present to match only directories,
+         * then it is stripped out by make_exclude.  So as a special
+         * case we add it back in here. */
+        
+        if (verbose >= 2)
+                rprintf(FINFO, "%s %s %s because of pattern %s%s\n",
+                        ent->include ? "including" : "excluding",
+                        S_ISDIR(st->st_mode) ? "directory" : "file",
+                        name, ent->pattern,
+                        ent->directory ? "/" : "");
+}
+
+
+/*
+ * Return true if file NAME is defined to be excluded by either
+ * LOCAL_EXCLUDE_LIST or the globals EXCLUDE_LIST.
+ */
+int check_exclude(char *name, struct exclude_struct **local_exclude_list,
 		  STRUCT_STAT *st)
 {
 	int n;
+        struct exclude_struct *ent;
 
 	if (name && (name[0] == '.') && !name[1])
 		/* never exclude '.', even if somebody does --exclude '*' */
 		return 0;
 
 	if (exclude_list) {
-		for (n=0; exclude_list[n]; n++)
-			if (check_one_exclude(name,exclude_list[n],st))
-				return !exclude_list[n]->include;
+		for (n=0; exclude_list[n]; n++) {
+                        ent = exclude_list[n];
+			if (check_one_exclude(name, ent, st)) {
+                                report_exclude_result(name, ent, st);
+				return !ent->include;
+                        }
+                }
 	}
 
 	if (local_exclude_list) {
-		for (n=0; local_exclude_list[n]; n++)
-			if (check_one_exclude(name,local_exclude_list[n],st))
-				return !local_exclude_list[n]->include;
+		for (n=0; local_exclude_list[n]; n++) {
+                        ent = local_exclude_list[n];
+			if (check_one_exclude(name, ent, st)) {
+                                report_exclude_result(name, ent, st);
+				return !ent->include;
+                        }
+                }
 	}
 
 	return 0;
 }
 
 
-void add_exclude_list(char *pattern,struct exclude_struct ***list, int include)
+void add_exclude_list(const char *pattern, struct exclude_struct ***list, int include)
 {
 	int len=0;
 	if (list && *list)
@@ -168,18 +201,20 @@ void add_exclude_list(char *pattern,struct exclude_struct ***list, int include)
 	if (!*list || !((*list)[len] = make_exclude(pattern, include)))
 		out_of_memory("add_exclude");
 	
-	if (verbose > 2)
-		rprintf(FINFO,"add_exclude(%s)\n",pattern);
-	
+	if (verbose > 2) {
+		rprintf(FINFO,"add_exclude(%s,%s)\n",pattern,
+			      include ? "include" : "exclude");
+	}
+
 	(*list)[len+1] = NULL;
 }
 
-void add_exclude(char *pattern, int include)
+void add_exclude(const char *pattern, int include)
 {
 	add_exclude_list(pattern,&exclude_list, include);
 }
 
-struct exclude_struct **make_exclude_list(char *fname,
+struct exclude_struct **make_exclude_list(const char *fname,
 					  struct exclude_struct **list1,
 					  int fatal, int include)
 {
@@ -188,7 +223,10 @@ struct exclude_struct **make_exclude_list(char *fname,
 	char line[MAXPATHLEN];
 	if (!f) {
 		if (fatal) {
-			rprintf(FERROR,"%s : %s\n",fname,strerror(errno));
+			rsyserr(FERROR, errno,
+                                "failed to open %s file %s",
+                                include ? "include" : "exclude",
+                                fname);
 			exit_cleanup(RERR_FILEIO);
 		}
 		return list;
@@ -210,7 +248,7 @@ struct exclude_struct **make_exclude_list(char *fname,
 }
 
 
-void add_exclude_file(char *fname,int fatal,int include)
+void add_exclude_file(const char *fname, int fatal, int include)
 {
 	if (!fname || !*fname) return;
 
@@ -222,6 +260,15 @@ void send_exclude_list(int f)
 {
 	int i;
 	extern int remote_version;
+	extern int list_only, recurse;
+
+	/* This is a complete hack - blame Rusty.
+	 *
+	 * FIXME: This pattern shows up in the output of
+	 * report_exclude_result(), which is not ideal. */
+	if (list_only && !recurse) {
+		add_exclude("/*/*", 0);
+	}
 
 	if (!exclude_list) {
 		write_int(f,0);
@@ -257,7 +304,8 @@ void send_exclude_list(int f)
 void recv_exclude_list(int f)
 {
 	char line[MAXPATHLEN];
-	int l;
+	unsigned int l;
+
 	while ((l=read_int(f))) {
 		if (l >= MAXPATHLEN) overflow("recv_exclude_list");
 		read_sbuf(f,line,l);
@@ -356,7 +404,7 @@ void add_cvs_excludes(void)
 		add_exclude(cvs_ignore_list[i], 0);
 
 	if ((p=getenv("HOME")) && strlen(p) < (MAXPATHLEN-12)) {
-		slprintf(fname,sizeof(fname), "%s/.cvsignore",p);
+		snprintf(fname,sizeof(fname), "%s/.cvsignore",p);
 		add_exclude_file(fname,0,0);
 	}
 
