@@ -101,12 +101,13 @@ static void finish_filelist_progress(const struct file_list *flist)
 {
 	if (do_progress) {
 		/* This overwrites the progress line */
-		rprintf(FINFO, "%d files to consider\n", flist->count);
-	} else
+		rprintf(FINFO, "%d file%sto consider\n",
+			flist->count, flist->count == 1 ? " " : "s ");
+	} else {
 		rprintf(FINFO, "done\n");
+	}
 }
 
-
 void show_flist_stats(void)
 {
 	/* Nothing yet */
@@ -987,8 +988,9 @@ struct file_list *send_file_list(int f, int argc, char *argv[])
 		send_file_entry(NULL, f, 0);
 	}
 
-	if (show_filelist_p())
+	if (show_filelist_p() && f != -1) {
 		finish_filelist_progress(flist);
+	}
 
 	clean_flist(flist, 0);
 
