@@ -22,7 +22,20 @@
 #include "popt.h"
 
 int make_backups = 0;
-int whole_file = -1;
+
+/**
+ * Should we send the whole file as literal data rather than trying to
+ * create an incremental diff?  This is on by default when both source
+ * and destination are local and we're not doing a batch delta,
+ * because there it's no cheaper to read the whole basis file than to
+ * just rewrite it.
+ *
+ * If both are 0, then look at whether we're local or remote and go by
+ * that.
+ **/
+int whole_file = 0;
+int no_whole_file = 0;
+
 int copy_links = 0;
 int preserve_links = 0;
 int preserve_hard_links = 0;
@@ -146,20 +159,28 @@ static void print_rsync_version(enum logcode f)
                 "Copyright (C) 1996-2002 by Andrew Tridgell and others\n");
 	rprintf(f, "<http://rsync.samba.org/>\n");
         rprintf(f, "Capabilities: %d-bit files, %ssocketpairs, "
-                "%shard links, %ssymlinks, batchfiles, %sIPv6,\n",
+                "%shard links, %ssymlinks, batchfiles, \n",
                 (int) (sizeof(OFF_T) * 8),
-                got_socketpair, hardlinks, links, ipv6);
+                got_socketpair, hardlinks, links);
 
 	/* Note that this field may not have type ino_t.  It depends
 	 * on the complicated interaction between largefile feature
 	 * macros. */
-	rprintf(f, "              %d-bit system inums, %d-bit internal inums\n",
+	rprintf(f, "              %sIPv6, %d-bit system inums, %d-bit internal inums\n",
+		ipv6, 
 		(int) (sizeof(dumstat->st_ino) * 8),
 		(int) (sizeof(INO64_T) * 8));
 
 #ifdef NO_INT64
         rprintf(f, "WARNING: no 64-bit integers on this platform!\n");
 #endif
+
+	rprintf(f,
+"\n"
+"rsync comes with ABSOLUTELY NO WARRANTY.  This is free software, and you\n"
+"are welcome to redistribute it under certain conditions.  See the GNU\n"
+"General Public Licence for details.\n"
+		);
 }
 
 
@@ -167,7 +188,7 @@ void usage(enum logcode F)
 {
   print_rsync_version(F);
 
-  rprintf(F,"rsync is a file transfer program capable of efficient remote update\nvia a fast differencing algorithm.\n\n");
+  rprintf(F,"\nrsync is a file transfer program capable of efficient remote update\nvia a fast differencing algorithm.\n\n");
 
   rprintf(F,"Usage: rsync [OPTION]... SRC [SRC]... [USER@]HOST:DEST\n");
   rprintf(F,"  or   rsync [OPTION]... [USER@]HOST:SRC DEST\n");
@@ -265,86 +286,86 @@ enum {OPT_VERSION = 1000, OPT_SUFFIX, OPT_SENDER, OPT_SERVER, OPT_EXCLUDE,
       OPT_LOG_FORMAT, OPT_PASSWORD_FILE, OPT_SIZE_ONLY, OPT_ADDRESS,
       OPT_DELETE_AFTER, OPT_EXISTING, OPT_MAX_DELETE, OPT_BACKUP_DIR, 
       OPT_IGNORE_ERRORS, OPT_BWLIMIT, OPT_BLOCKING_IO,
-      OPT_NO_BLOCKING_IO, OPT_NO_WHOLE_FILE,
+      OPT_NO_BLOCKING_IO, OPT_WHOLE_FILE, OPT_NO_WHOLE_FILE,
       OPT_MODIFY_WINDOW, OPT_READ_BATCH, OPT_WRITE_BATCH, OPT_IGNORE_EXISTING};
 
 static struct poptOption long_options[] = {
   /* longName, shortName, argInfo, argPtr, value, descrip, argDesc */
-  {"version",          0,  POPT_ARG_NONE,   0,             OPT_VERSION},
-  {"suffix",           0,  POPT_ARG_STRING, &backup_suffix},
-  {"rsync-path",       0,  POPT_ARG_STRING, &rsync_path},
-  {"password-file",    0,  POPT_ARG_STRING, &password_file},
-  {"ignore-times",    'I', POPT_ARG_NONE,   &ignore_times},
-  {"size-only",        0,  POPT_ARG_NONE,   &size_only},
-  {"modify-window",    0,  POPT_ARG_INT,    &modify_window, OPT_MODIFY_WINDOW},
-  {"one-file-system", 'x', POPT_ARG_NONE,   &one_file_system},
-  {"delete",           0,  POPT_ARG_NONE,   &delete_mode},
-  {"existing",         0,  POPT_ARG_NONE,   &only_existing},
-  {"ignore-existing",  0,  POPT_ARG_NONE,   &opt_ignore_existing},
-  {"delete-after",     0,  POPT_ARG_NONE,   &delete_after},
-  {"delete-excluded",  0,  POPT_ARG_NONE,   0,              OPT_DELETE_EXCLUDED},
-  {"force",            0,  POPT_ARG_NONE,   &force_delete},
-  {"numeric-ids",      0,  POPT_ARG_NONE,   &numeric_ids},
-  {"exclude",          0,  POPT_ARG_STRING, 0,              OPT_EXCLUDE},
-  {"include",          0,  POPT_ARG_STRING, 0,              OPT_INCLUDE},
-  {"exclude-from",     0,  POPT_ARG_STRING, 0,              OPT_EXCLUDE_FROM},
-  {"include-from",     0,  POPT_ARG_STRING, 0,              OPT_INCLUDE_FROM},
-  {"safe-links",       0,  POPT_ARG_NONE,   &safe_symlinks},
-  {"help",            'h', POPT_ARG_NONE,   0,              'h'},
-  {"backup",          'b', POPT_ARG_NONE,   &make_backups},
-  {"dry-run",         'n', POPT_ARG_NONE,   &dry_run},
-  {"sparse",          'S', POPT_ARG_NONE,   &sparse_files},
-  {"cvs-exclude",     'C', POPT_ARG_NONE,   &cvs_exclude},
-  {"update",          'u', POPT_ARG_NONE,   &update_only},
-  {"links",           'l', POPT_ARG_NONE,   &preserve_links},
-  {"copy-links",      'L', POPT_ARG_NONE,   &copy_links},
-  {"whole-file",      'W', POPT_ARG_NONE,   &whole_file},
-  {"no-whole-file",    0,  POPT_ARG_NONE,   0, 		     OPT_NO_WHOLE_FILE},
-  {"copy-unsafe-links", 0, POPT_ARG_NONE,   &copy_unsafe_links},
-  {"perms",           'p', POPT_ARG_NONE,   &preserve_perms},
-  {"owner",           'o', POPT_ARG_NONE,   &preserve_uid},
-  {"group",           'g', POPT_ARG_NONE,   &preserve_gid},
-  {"devices",         'D', POPT_ARG_NONE,   &preserve_devices},
-  {"times",           't', POPT_ARG_NONE,   &preserve_times},
-  {"checksum",        'c', POPT_ARG_NONE,   &always_checksum},
-  {"verbose",         'v', POPT_ARG_NONE,   0,               'v'},
-  {"quiet",           'q', POPT_ARG_NONE,   0,               'q'},
-  {"archive",         'a', POPT_ARG_NONE,   0,               'a'}, 
-  {"server",           0,  POPT_ARG_NONE,   &am_server},
-  {"sender",           0,  POPT_ARG_NONE,   0,               OPT_SENDER},
-  {"recursive",       'r', POPT_ARG_NONE,   &recurse},
-  {"relative",        'R', POPT_ARG_NONE,   &relative_paths},
-  {"rsh",             'e', POPT_ARG_STRING, &shell_cmd},
-  {"block-size",      'B', POPT_ARG_INT,    &block_size},
-  {"max-delete",       0,  POPT_ARG_INT,    &max_delete},
-  {"timeout",          0,  POPT_ARG_INT,    &io_timeout},
-  {"temp-dir",        'T', POPT_ARG_STRING, &tmpdir},
-  {"compare-dest",     0,  POPT_ARG_STRING, &compare_dest},
+  {"version",          0,  POPT_ARG_NONE,   0,             OPT_VERSION, 0, 0},
+  {"suffix",           0,  POPT_ARG_STRING, &backup_suffix,	0, 0, 0 },
+  {"rsync-path",       0,  POPT_ARG_STRING, &rsync_path,	0, 0, 0 },
+  {"password-file",    0,  POPT_ARG_STRING, &password_file,	0, 0, 0 },
+  {"ignore-times",    'I', POPT_ARG_NONE,   &ignore_times , 0, 0, 0 },
+  {"size-only",        0,  POPT_ARG_NONE,   &size_only , 0, 0, 0 },
+  {"modify-window",    0,  POPT_ARG_INT,    &modify_window, OPT_MODIFY_WINDOW, 0, 0 },
+  {"one-file-system", 'x', POPT_ARG_NONE,   &one_file_system , 0, 0, 0 },
+  {"delete",           0,  POPT_ARG_NONE,   &delete_mode , 0, 0, 0 },
+  {"existing",         0,  POPT_ARG_NONE,   &only_existing , 0, 0, 0 },
+  {"ignore-existing",  0,  POPT_ARG_NONE,   &opt_ignore_existing , 0, 0, 0 },
+  {"delete-after",     0,  POPT_ARG_NONE,   &delete_after , 0, 0, 0 },
+  {"delete-excluded",  0,  POPT_ARG_NONE,   0,              OPT_DELETE_EXCLUDED, 0, 0 },
+  {"force",            0,  POPT_ARG_NONE,   &force_delete , 0, 0, 0 },
+  {"numeric-ids",      0,  POPT_ARG_NONE,   &numeric_ids , 0, 0, 0 },
+  {"exclude",          0,  POPT_ARG_STRING, 0,              OPT_EXCLUDE, 0, 0 },
+  {"include",          0,  POPT_ARG_STRING, 0,              OPT_INCLUDE, 0, 0 },
+  {"exclude-from",     0,  POPT_ARG_STRING, 0,              OPT_EXCLUDE_FROM, 0, 0 },
+  {"include-from",     0,  POPT_ARG_STRING, 0,              OPT_INCLUDE_FROM, 0, 0 },
+  {"safe-links",       0,  POPT_ARG_NONE,   &safe_symlinks , 0, 0, 0 },
+  {"help",            'h', POPT_ARG_NONE,   0,              'h', 0, 0 },
+  {"backup",          'b', POPT_ARG_NONE,   &make_backups , 0, 0, 0 },
+  {"dry-run",         'n', POPT_ARG_NONE,   &dry_run , 0, 0, 0 },
+  {"sparse",          'S', POPT_ARG_NONE,   &sparse_files , 0, 0, 0 },
+  {"cvs-exclude",     'C', POPT_ARG_NONE,   &cvs_exclude , 0, 0, 0 },
+  {"update",          'u', POPT_ARG_NONE,   &update_only , 0, 0, 0 },
+  {"links",           'l', POPT_ARG_NONE,   &preserve_links , 0, 0, 0 },
+  {"copy-links",      'L', POPT_ARG_NONE,   &copy_links , 0, 0, 0 },
+  {"whole-file",      'W', POPT_ARG_NONE,   0,              OPT_WHOLE_FILE, 0, 0 },
+  {"no-whole-file",    0,  POPT_ARG_NONE,   0,              OPT_NO_WHOLE_FILE, 0, 0 },
+  {"copy-unsafe-links", 0, POPT_ARG_NONE,   &copy_unsafe_links , 0, 0, 0 },
+  {"perms",           'p', POPT_ARG_NONE,   &preserve_perms , 0, 0, 0 },
+  {"owner",           'o', POPT_ARG_NONE,   &preserve_uid , 0, 0, 0 },
+  {"group",           'g', POPT_ARG_NONE,   &preserve_gid , 0, 0, 0 },
+  {"devices",         'D', POPT_ARG_NONE,   &preserve_devices , 0, 0, 0 },
+  {"times",           't', POPT_ARG_NONE,   &preserve_times , 0, 0, 0 },
+  {"checksum",        'c', POPT_ARG_NONE,   &always_checksum , 0, 0, 0 },
+  {"verbose",         'v', POPT_ARG_NONE,   0,               'v', 0, 0 },
+  {"quiet",           'q', POPT_ARG_NONE,   0,               'q', 0, 0 },
+  {"archive",         'a', POPT_ARG_NONE,   0,               'a', 0, 0 }, 
+  {"server",           0,  POPT_ARG_NONE,   &am_server , 0, 0, 0 },
+  {"sender",           0,  POPT_ARG_NONE,   0,               OPT_SENDER, 0, 0 },
+  {"recursive",       'r', POPT_ARG_NONE,   &recurse , 0, 0, 0 },
+  {"relative",        'R', POPT_ARG_NONE,   &relative_paths , 0, 0, 0 },
+  {"rsh",             'e', POPT_ARG_STRING, &shell_cmd , 0, 0, 0 },
+  {"block-size",      'B', POPT_ARG_INT,    &block_size , 0, 0, 0 },
+  {"max-delete",       0,  POPT_ARG_INT,    &max_delete , 0, 0, 0 },
+  {"timeout",          0,  POPT_ARG_INT,    &io_timeout , 0, 0, 0 },
+  {"temp-dir",        'T', POPT_ARG_STRING, &tmpdir , 0, 0, 0 },
+  {"compare-dest",     0,  POPT_ARG_STRING, &compare_dest , 0, 0, 0 },
   /* TODO: Should this take an optional int giving the compression level? */
-  {"compress",        'z', POPT_ARG_NONE,   &do_compression},
-  {"daemon",           0,  POPT_ARG_NONE,   &am_daemon},
-  {"no-detach",        0,  POPT_ARG_NONE,   &no_detach},
-  {"stats",            0,  POPT_ARG_NONE,   &do_stats},
-  {"progress",         0,  POPT_ARG_NONE,   &do_progress},
-  {"partial",          0,  POPT_ARG_NONE,   &keep_partial},
-  {"ignore-errors",    0,  POPT_ARG_NONE,   &ignore_errors},
-  {"blocking-io",      0,  POPT_ARG_NONE,   &blocking_io},
-  {"no-blocking-io",   0,  POPT_ARG_NONE,   0, 		     OPT_NO_BLOCKING_IO},
-  {0,                 'P', POPT_ARG_NONE,   0,               'P'},
-  {"config",           0,  POPT_ARG_STRING, &config_file},
-  {"port",             0,  POPT_ARG_INT,    &rsync_port},
-  {"log-format",       0,  POPT_ARG_STRING, &log_format},
-  {"bwlimit",          0,  POPT_ARG_INT,    &bwlimit},
-  {"address",          0,  POPT_ARG_STRING, &bind_address, 0},
-  {"backup-dir",       0,  POPT_ARG_STRING, &backup_dir},
-  {"hard-links",      'H', POPT_ARG_NONE,   &preserve_hard_links},
-  {"read-batch",       0,  POPT_ARG_STRING, &batch_prefix, OPT_READ_BATCH},
-  {"write-batch",      0,  POPT_ARG_STRING, &batch_prefix, OPT_WRITE_BATCH},
+  {"compress",        'z', POPT_ARG_NONE,   &do_compression , 0, 0, 0 },
+  {"daemon",           0,  POPT_ARG_NONE,   &am_daemon , 0, 0, 0 },
+  {"no-detach",        0,  POPT_ARG_NONE,   &no_detach , 0, 0, 0 },
+  {"stats",            0,  POPT_ARG_NONE,   &do_stats , 0, 0, 0 },
+  {"progress",         0,  POPT_ARG_NONE,   &do_progress , 0, 0, 0 },
+  {"partial",          0,  POPT_ARG_NONE,   &keep_partial , 0, 0, 0 },
+  {"ignore-errors",    0,  POPT_ARG_NONE,   &ignore_errors , 0, 0, 0 },
+  {"blocking-io",      0,  POPT_ARG_NONE,   &blocking_io , 0, 0, 0 },
+  {"no-blocking-io",   0,  POPT_ARG_NONE,   0, 		     OPT_NO_BLOCKING_IO, 0, 0 },
+  {0,                 'P', POPT_ARG_NONE,   0,               'P', 0, 0 },
+  {"config",           0,  POPT_ARG_STRING, &config_file , 0, 0, 0 },
+  {"port",             0,  POPT_ARG_INT,    &rsync_port , 0, 0, 0 },
+  {"log-format",       0,  POPT_ARG_STRING, &log_format , 0, 0, 0 },
+  {"bwlimit",          0,  POPT_ARG_INT,    &bwlimit , 0, 0, 0 },
+  {"address",          0,  POPT_ARG_STRING, &bind_address, 0, 0, 0 },
+  {"backup-dir",       0,  POPT_ARG_STRING, &backup_dir , 0, 0, 0 },
+  {"hard-links",      'H', POPT_ARG_NONE,   &preserve_hard_links , 0, 0, 0 },
+  {"read-batch",       0,  POPT_ARG_STRING, &batch_prefix, OPT_READ_BATCH, 0, 0 },
+  {"write-batch",      0,  POPT_ARG_STRING, &batch_prefix, OPT_WRITE_BATCH, 0, 0 },
 #ifdef INET6
-  {0,		      '4', POPT_ARG_VAL,    &default_af_hint,   AF_INET },
-  {0,		      '6', POPT_ARG_VAL,    &default_af_hint,   AF_INET6 },
+  {0,		      '4', POPT_ARG_VAL,    &default_af_hint,   AF_INET , 0, 0 },
+  {0,		      '6', POPT_ARG_VAL,    &default_af_hint,   AF_INET6 , 0, 0 },
 #endif
-  {0,0,0,0}
+  {0,0,0,0, 0, 0, 0}
 };
 
 
@@ -463,7 +484,13 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 			add_exclude_file(poptGetOptArg(pc), 1, 1);
 			break;
 
+		case OPT_WHOLE_FILE:
+			whole_file = 1;
+			no_whole_file = 0;
+			break;
+
 		case OPT_NO_WHOLE_FILE:
+			no_whole_file = 1;
 			whole_file = 0;
 			break;
 
@@ -590,8 +617,6 @@ void server_options(char **args,int *argc)
 
 	int i, x;
 
-	if (whole_file == -1)
-		whole_file = 0;
 	if (blocking_io == -1)
 		blocking_io = 0;
 
@@ -616,8 +641,14 @@ void server_options(char **args,int *argc)
 		argstr[x++] = 'l';
 	if (copy_links)
 		argstr[x++] = 'L';
+
+	assert(whole_file == 0 || whole_file == 1);
 	if (whole_file)
 		argstr[x++] = 'W';
+	/* We don't need to send --no-whole-file, because it's the
+	 * default for remote transfers, and in any case old versions
+	 * of rsync will not understand it. */
+	
 	if (preserve_hard_links)
 		argstr[x++] = 'H';
 	if (preserve_uid)
@@ -758,7 +789,6 @@ void server_options(char **args,int *argc)
 		args[ac++] = compare_dest;
 	}
 
-
 	*argc = ac;
 }
 
