@@ -125,6 +125,9 @@ set -e
 
 RUNSHFLAGS='-e'
 
+# for Solaris
+PATH="/usr/xpg4/bin/:$PATH"
+
 if [ -n "$loglevel" ] && [ "$loglevel" -gt 8 ]
 then
     if set -x
@@ -139,6 +142,20 @@ echo "$0 running in `pwd`"
 echo "    rsync_bin=$rsync_bin"
 echo "    srcdir=$srcdir"
 
+testuser=`whoami || echo UNKNOWN`
+
+echo "    testuser=$testuser"
+echo "    os=`uname -a`"
+
+# It must be "yes", not just nonnull
+if test "x$preserve_scratch" = xyes
+then
+    echo "    preserve_scratch=yes"
+else
+    echo "    preserve_scratch=no"
+fi    
+
+
 if test ! -f $rsync_bin
 then
     echo "rsync_bin $rsync_bin is not a file" >&2
@@ -212,7 +229,8 @@ do
 	maybe_discard_scratch
 	;;
     77)
-	echo "SKIP    $testbase"
+	# backticks will fill the whole file onto one line, which is a feature
+	echo "SKIP    $testbase (`cat \"$scratchdir/whyskipped\"`)"
 	skipped=`expr $skipped + 1`
 	maybe_discard_scratch
 	;;
